var FormController = function($scope, eventEngine) {

	var self = this;

	this.__defineGetter__('widget', function() {
		return $scope.widget;
	});

	// Floating Control defaults
	var floatingControlDefaults = {
		pdfAction: false,
		xlsAction: false,
		searchAction: false,
		csvAction: false,
		customizationAction: {
			filter: false,
			layout: false,
			view: false
		}
	};

	self.widget.floatingControl = self.widget.floatingControl !== false && Util.extend(floatingControlDefaults, self.widget.floatingControl || {});

	if (!self.widget.afterBuildFieldGroups) {
		eventEngine.createEvent(self.widget, "afterBuildFieldGroups");
	}

	this.isLabelField = function(field){
		return field && field.template === 'field/label.html';
	};

	var lookupTable = {
		'widget/form.html': "showOnForm",
		'widget/form-view.html': "showOnView",
		'widget/form-edit.html': "showOnForm",
		'widget/form-structure.html': "showOnForm",
		'widget/form-view-edit.html': "showOnForm",
		'widget/form-without-scholler.html': "showOnForm",
		'widget/form-without-scroller.html': "showOnForm",
		'widget/form-grid.html': "showOnView",
		'widget/form-login.html': "showOnForm",
		'widget/form-center-top.html': "showOnForm",
		"widget/stripe.html": "showOnForm",
		"widget/stripe-collapsible.html": "showOnForm",
		"widget/navigation-stripe.html": "showOnForm"
	};

	function visibleOnTemplate(field) {
		var template = field.widget.childTemplate || field.widget.formTemplate || field.widget.template;
		var isVisible = false;
		if(template) {
			isVisible = field[lookupTable[template.slice(template.indexOf('widget'))]];
		}
		return isVisible;
	}

	function orderFieldsBySequenceInForm(){
		self.widget.fields = Util.mergeSort(self.widget.fields, function(firstField, secondField){
			return firstField.sequenceInForm - secondField.sequenceInForm;
		});
	}

	function groupFieldsByFieldGroup(widgetFields, widgetFieldGroups) {
		orderFieldsBySequenceInForm();
		if (widgetFields && widgetFields.length > 0) {

			if (widgetFieldGroups && widgetFieldGroups.length > 0) {
				widgetFieldGroups.forEach(function(fieldGroup) {
					if (fieldGroup.name) {
						var fields = widgetFields.filter(function(field) {
							return field.isVisible && visibleOnTemplate(field) && !Util.isUndefinedOrNull(field.fieldGroup) && field.fieldGroup === fieldGroup.name;
						});

						fieldGroup._hasVisibleFields = fields.length > 0;
						fieldGroup.rows = groupFieldsByRowSize(fields, "class", 12);
					}
				});
			}

			var noFieldGroupInputs = widgetFields.filter(function(field) {
				return !field.fieldGroup && visibleOnTemplate(field);
			});

			if (noFieldGroupInputs.length > 0) {

				if (!Array.isArray(widgetFieldGroups)) {
					widgetFieldGroups = [];
				}

				var defaultFieldGroup = new FieldGroup();
				defaultFieldGroup.rows = groupFieldsByRowSize(noFieldGroupInputs, "class", 12);
				defaultFieldGroup._hasVisibleFields = defaultFieldGroup.rows[0].fields.length > 0;

				if (self.widget.fieldGroups && !self.widget.fieldGroups[0].name) {
					self.widget.fieldGroups.splice(0, 1);
				}

				widgetFieldGroups.unshift(defaultFieldGroup);
			}
		} else {
			if (widgetFieldGroups && widgetFieldGroups.length > 0) {
				widgetFieldGroups.forEach(function(fieldGroup) {
					fieldGroup._hasVisibleFields = false;
				});
			}
		}

		return widgetFieldGroups;
	}

	function getFieldSize(property) {
		var windowSize = window.innerWidth;
		var result, size;
		var sizes = { xs: 0 };

		property.toString().split(' ').forEach(function(i) {
			if (i.indexOf('col-sm-') !== -1) {
				sizes.sm = parseInt(i.substr(7),10);
			} else if (i.indexOf('col-md-') !== -1) {
				sizes.md = parseInt(i.substr(7),10);
			} else if (i.indexOf('col-lg-') !== -1) {
				sizes.lg = parseInt(i.substr(7),10);
			} else if (i.indexOf('offset') === -1) {
				size = parseInt(i,10);
				if (!isNaN(size)) {
					sizes.xs = size;
				}
			}
		});

		if (windowSize >= 1200 && !Util.isUndefined(sizes.lg)) {
			result = sizes.lg;
		} else if (windowSize >= 992 && !Util.isUndefined(sizes.md)) {
			result = sizes.md;
		} else if (windowSize >= 768 && !Util.isUndefined(sizes.sm)) {
			result = sizes.sm;
		} else {
			result = sizes.xs;
		}

		return result;
	}

	function groupFieldsByRowSize(fields, property, maxSize) {
		var rows = [{
			fields: []
		}];
		var rowSize = 0;
		var colSize;
		for (var i = 0; i < fields.length; i++) {
			if(fields[i].isVisible && fields[i].aclIsVisible){
				colSize = getFieldSize(fields[i][property]);
				if (colSize > maxSize) {
					colSize = maxSize;
					fields[i][property] = colSize;
				}

				rowSize += colSize;
				if (rowSize > maxSize || fields[i].breakline && rows[rows.length - 1].fields.length > 0) {
					rows.push({
						fields: []
					});
					rowSize = colSize;
				}
				rows[rows.length - 1].fields.push(fields[i]);

				//Reset fields lastOfRow to false
				fields[i].lastOfRow = false;

				if (rowSize == maxSize) {
					fields[i].lastOfRow = true;
				}
			}
		}
		return rows;
	}

	$scope.$watchCollection(function() {
		return self.widget.fields.filter(function(field) {
			return field.isVisible && field.aclIsVisible && (field.showOnForm || field.showOnView);
		});
	}, function(fields) {
		if (self.widget) {
			self.widget.fieldGroups = groupFieldsByFieldGroup(fields, self.widget.fieldGroups);
			publishAfterBuildFieldGroupsEvent();
		}
	});

	$scope.$watch('widget.template', function() {
		if (self.widget) {
			self.widget.fieldGroups = groupFieldsByFieldGroup(self.widget.fields, self.widget.fieldGroups);
			publishAfterBuildFieldGroupsEvent();
		}
	});

	var oldRedraw = $scope.widget.redraw;
	$scope.widget.redraw = function(){
		self.widget.fieldGroups = groupFieldsByFieldGroup(self.widget.fields, self.widget.fieldGroups);
		publishAfterBuildFieldGroupsEvent();
	};

	var _widget = self.widget;
	$scope.$on('$destroy', function() {
		_widget.redraw = oldRedraw;
	});

	function publishAfterBuildFieldGroupsEvent() {
		if (self.widget.afterBuildFieldGroups) {
			self.widget.afterBuildFieldGroups();
		}
	}

	if ($scope.widget.orderFieldsBySequenceInForm) {
		orderFieldsBySequenceInForm();
	}
};

ZeedhiController.controller('FormController', FormController);